@@ -7,28 +7,38 @@ from textual._resolve import resolve
 from textual.css.scalar import Scalar
 from textual.geometry import NULL_OFFSET, Region, Size, Spacing
 from textual.layout import ArrangeResult, Layout, WidgetPlacement
+from textual.visual import visualize
 
 if TYPE_CHECKING:
     from textual.widget import Widget
 
 
 class GridLayout(Layout):
-    """Used to layout Widgets in to a grid."""
+    """Used to layout Widgets into a grid."""
 
     name = "grid"
 
     def __init__(self) -> None:
         self.min_column_width: int | None = None
         self.stretch_height: bool = False
-        self.regular = False
+        """Stretch the height of cells to be equal in each row."""
+        self.regular: bool = False
+        self.expand: bool = False
+        """Expand the grid to fit the container if it is smaller."""
+        self.shrink: bool = False
+        """Shrink the grid to fit the container if it is larger."""
+        self.auto_minimum: bool = False
+        """If self.shrink is `True`, auto-detect and limit the width."""
 
     def arrange(
-        self, parent: Widget, children: list[Widget], size: Size
+        self, parent: Widget, children: list[Widget], size: Size, greedy: bool = True
     ) -> ArrangeResult:
         parent.pre_layout(self)
         styles = parent.styles
         row_scalars = styles.grid_rows or (
-            [Scalar.parse("1fr")] if size.height else [Scalar.parse("auto")]
+            [Scalar.parse("1fr")]
+            if (size.height and not parent.styles.is_auto_height)
+            else [Scalar.parse("auto")]
         )
         column_scalars = styles.grid_columns or [Scalar.parse("1fr")]
         gutter_horizontal = styles.grid_gutter_horizontal
@@ -50,7 +60,7 @@ class GridLayout(Layout):
                     table_size_columns -= 1
 
         table_size_rows = styles.grid_size_rows
-        viewport = parent.screen.size
+        viewport = parent.app.viewport_size
         keyline_style, _keyline_color = styles.keyline
         offset = (0, 0)
         gutter_spacing: Spacing | None
@@ -220,12 +230,32 @@ class GridLayout(Layout):
                         )
                 column_scalars[column] = Scalar.from_number(width)
 
+        column_minimums: list[int] | None = None
+        if self.auto_minimum and self.shrink:
+            column_minimums = [1] * table_size_columns
+            for column_index in range(table_size_columns):
+                for row_index in range(len(row_scalars)):
+                    if (
+                        cell_info := cell_map.get((column_index, row_index))
+                    ) is not None:
+                        widget = cell_info[0]
+                        column_minimums[column_index] = max(
+                            visualize(widget, widget.render()).get_minimal_width(
+                                widget.styles
+                            )
+                            + widget.styles.gutter.width,
+                            column_minimums[column_index],
+                        )
+
         columns = resolve(
             column_scalars,
             size.width,
             gutter_vertical,
             size,
             viewport,
+            expand=self.expand,
+            shrink=self.shrink,
+            minimums=column_minimums,
         )
 
         # Handle any auto rows
@@ -253,6 +283,7 @@ class GridLayout(Layout):
                             + gutter_height,
                         )
                         height = max(height, widget_height)
+
                 row_scalars[row] = Scalar.from_number(height)
 
         rows = resolve(row_scalars, size.height, gutter_horizontal, size, viewport)
@@ -271,12 +302,16 @@ class GridLayout(Layout):
             x2, cell_width = columns[min(max_column, column + column_span)]
             y2, cell_height = rows[min(max_row, row + row_span)]
             cell_size = Size(cell_width + x2 - x, cell_height + y2 - y)
+
             box_width, box_height, margin = widget._get_box_model(
                 cell_size,
                 viewport,
                 Fraction(cell_size.width),
                 Fraction(cell_size.height),
+                constrain_width=True,
+                greedy=greedy,
             )
+
             if self.stretch_height and len(children) > 1:
                 if box_height <= cell_size.height:
                     box_height = Fraction(cell_size.height)
